/*
 * BasicBattleSimulatorView.java
 */

package basicbattlesimulator;

import org.jdesktop.application.Action;
import org.jdesktop.application.ResourceMap;
import org.jdesktop.application.SingleFrameApplication;
import org.jdesktop.application.FrameView;
import org.jdesktop.application.TaskMonitor;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.*;

import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;
import java.util.concurrent.Executors;
import static java.util.concurrent.TimeUnit.*;

/**
 * The application's main frame.
 */
public class BasicBattleSimulatorView extends FrameView{
    
    //EquipProperties equip = new EquipProperties();
    //Battle battle;
    
    //dialogs and string arrays to pass along the properties associated with selections
    protected PlayerDialog playerDialog;
    protected String[] playerProperties = new String[7];
    protected EquipDialog equipDialog;
    protected String[] weaponProperties = new String[4];
    protected String[] bodyArmourProperties = new String[3];
    //protected PlayerCharacter player;
    //protected NonPlayerCharacter enemy;

    public BasicBattleSimulatorView(SingleFrameApplication app) {
        super(app);

        initComponents();

        // status bar initialization - message timeout, idle icon and busy animation, etc
        ResourceMap resourceMap = getResourceMap();
        int messageTimeout = resourceMap.getInteger("StatusBar.messageTimeout");
        messageTimer = new Timer(messageTimeout, new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                //statusMessageLabel.setText("");
            }
        });
        messageTimer.setRepeats(false);
        int busyAnimationRate = resourceMap.getInteger("StatusBar.busyAnimationRate");
        for (int i = 0; i < busyIcons.length; i++) {
            busyIcons[i] = resourceMap.getIcon("StatusBar.busyIcons[" + i + "]");
        }
        busyIconTimer = new Timer(busyAnimationRate, new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                busyIconIndex = (busyIconIndex + 1) % busyIcons.length;
                //statusAnimationLabel.setIcon(busyIcons[busyIconIndex]);
            }
        });
        idleIcon = resourceMap.getIcon("StatusBar.idleIcon");
        //statusAnimationLabel.setIcon(idleIcon);
        //progressBar.setVisible(false);

        // connecting action tasks to status bar via TaskMonitor
        TaskMonitor taskMonitor = new TaskMonitor(getApplication().getContext());
        taskMonitor.addPropertyChangeListener(new java.beans.PropertyChangeListener() {
            public void propertyChange(java.beans.PropertyChangeEvent evt) {
                String propertyName = evt.getPropertyName();
                if ("started".equals(propertyName)) {
                    if (!busyIconTimer.isRunning()) {
                        //statusAnimationLabel.setIcon(busyIcons[0]);
                        busyIconIndex = 0;
                        busyIconTimer.start();
                    }
                    //progressBar.setVisible(true);
                    //progressBar.setIndeterminate(true);
                } else if ("done".equals(propertyName)) {
                    busyIconTimer.stop();
                    //statusAnimationLabel.setIcon(idleIcon);
                    //progressBar.setVisible(false);
                    //progressBar.setValue(0);
                } else if ("message".equals(propertyName)) {
                    String text = (String)(evt.getNewValue());
                    //statusMessageLabel.setText((text == null) ? "" : text);
                    messageTimer.restart();
                } else if ("progress".equals(propertyName)) {
                    int value = (Integer)(evt.getNewValue());
                    //progressBar.setVisible(true);
                    //progressBar.setIndeterminate(false);
                    //progressBar.setValue(value);
                }
            }
        });
    }

    @Action
    public void showAboutBox() {
        if (aboutBox == null) {
            JFrame mainFrame = BasicBattleSimulatorApp.getApplication().getMainFrame();
            aboutBox = new BasicBattleSimulatorAboutBox(mainFrame);
            aboutBox.setLocationRelativeTo(mainFrame);
        }
        BasicBattleSimulatorApp.getApplication().show(aboutBox);
    }

    @Action
    public void showEquipDialog() {
        if (equipDialog == null) {
            JFrame mainFrame = BasicBattleSimulatorApp.getApplication().getMainFrame();
            equipDialog = new EquipDialog(mainFrame, true);
            equipDialog.setLocationRelativeTo(mainFrame);
        }
        BasicBattleSimulatorApp.getApplication().show(equipDialog);
    }
    
    @Action
    public void showPlayerDialog() {
        if (playerDialog == null) {
            JFrame mainFrame = BasicBattleSimulatorApp.getApplication().getMainFrame();
            playerDialog = new PlayerDialog(mainFrame, true);
            playerDialog.setLocationRelativeTo(mainFrame);
        }
        BasicBattleSimulatorApp.getApplication().show(playerDialog);
    }    
  
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        mainPanel = new javax.swing.JPanel();
        jScrollPane1 = new javax.swing.JScrollPane();
        battleReportTextArea = new javax.swing.JTextArea();
        jButton1 = new javax.swing.JButton();
        jButton2 = new javax.swing.JButton();
        jPanel1 = new javax.swing.JPanel();
        enemyImage = new javax.swing.JLabel();
        jPanel2 = new javax.swing.JPanel();
        playerImage = new javax.swing.JLabel();
        jButton3 = new javax.swing.JButton();
        menuBar = new javax.swing.JMenuBar();
        javax.swing.JMenu fileMenu = new javax.swing.JMenu();
        javax.swing.JMenuItem exitMenuItem = new javax.swing.JMenuItem();
        jMenu1 = new javax.swing.JMenu();
        jMenuItem2 = new javax.swing.JMenuItem();
        jMenuItem1 = new javax.swing.JMenuItem();
        javax.swing.JMenu helpMenu = new javax.swing.JMenu();
        javax.swing.JMenuItem aboutMenuItem = new javax.swing.JMenuItem();

        mainPanel.setMaximumSize(new java.awt.Dimension(683, 470));
        mainPanel.setMinimumSize(new java.awt.Dimension(683, 470));
        mainPanel.setName("mainPanel"); // NOI18N

        jScrollPane1.setName("jScrollPane1"); // NOI18N

        battleReportTextArea.setColumns(20);
        battleReportTextArea.setEditable(false);
        battleReportTextArea.setLineWrap(true);
        battleReportTextArea.setRows(5);
        org.jdesktop.application.ResourceMap resourceMap = org.jdesktop.application.Application.getInstance(basicbattlesimulator.BasicBattleSimulatorApp.class).getContext().getResourceMap(BasicBattleSimulatorView.class);
        battleReportTextArea.setToolTipText(resourceMap.getString("battleReportTextArea.toolTipText")); // NOI18N
        battleReportTextArea.setName("battleReportTextArea"); // NOI18N
        jScrollPane1.setViewportView(battleReportTextArea);

        jButton1.setText(resourceMap.getString("jButton1.text")); // NOI18N
        jButton1.setToolTipText(resourceMap.getString("jButton1.toolTipText")); // NOI18N
        jButton1.setName("jButton1"); // NOI18N
        jButton1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton1ActionPerformed(evt);
            }
        });

        javax.swing.ActionMap actionMap = org.jdesktop.application.Application.getInstance(basicbattlesimulator.BasicBattleSimulatorApp.class).getContext().getActionMap(BasicBattleSimulatorView.class, this);
        jButton2.setAction(actionMap.get("quit")); // NOI18N
        jButton2.setText(resourceMap.getString("jButton2.text")); // NOI18N
        jButton2.setToolTipText(resourceMap.getString("jButton2.toolTipText")); // NOI18N
        jButton2.setName("jButton2"); // NOI18N

        jPanel1.setBorder(javax.swing.BorderFactory.createTitledBorder(resourceMap.getString("jPanel1.border.title"))); // NOI18N
        jPanel1.setName("jPanel1"); // NOI18N
        jPanel1.setPreferredSize(new java.awt.Dimension(325, 208));

        enemyImage.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        enemyImage.setIcon(resourceMap.getIcon("enemyImage.icon")); // NOI18N
        enemyImage.setText(resourceMap.getString("enemyImage.text")); // NOI18N
        enemyImage.setName("enemyImage"); // NOI18N

        javax.swing.GroupLayout jPanel1Layout = new javax.swing.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(enemyImage, javax.swing.GroupLayout.PREFERRED_SIZE, 289, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap())
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addComponent(enemyImage, javax.swing.GroupLayout.PREFERRED_SIZE, 173, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap())
        );

        jPanel2.setBorder(javax.swing.BorderFactory.createTitledBorder(resourceMap.getString("jPanel2.border.title"))); // NOI18N
        jPanel2.setName("jPanel2"); // NOI18N
        jPanel2.setPreferredSize(new java.awt.Dimension(325, 208));

        playerImage.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        playerImage.setIcon(resourceMap.getIcon("playerImage.icon")); // NOI18N
        playerImage.setText(resourceMap.getString("playerImage.text")); // NOI18N
        playerImage.setName("playerImage"); // NOI18N

        javax.swing.GroupLayout jPanel2Layout = new javax.swing.GroupLayout(jPanel2);
        jPanel2.setLayout(jPanel2Layout);
        jPanel2Layout.setHorizontalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel2Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(playerImage, javax.swing.GroupLayout.PREFERRED_SIZE, 289, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap())
        );
        jPanel2Layout.setVerticalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel2Layout.createSequentialGroup()
                .addComponent(playerImage, javax.swing.GroupLayout.PREFERRED_SIZE, 173, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap())
        );

        jButton3.setText(resourceMap.getString("jButton3.text")); // NOI18N
        jButton3.setToolTipText(resourceMap.getString("jButton3.toolTipText")); // NOI18N
        jButton3.setName("jButton3"); // NOI18N
        jButton3.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton3ActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout mainPanelLayout = new javax.swing.GroupLayout(mainPanel);
        mainPanel.setLayout(mainPanelLayout);
        mainPanelLayout.setHorizontalGroup(
            mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(mainPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(mainPanelLayout.createSequentialGroup()
                        .addGroup(mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 663, Short.MAX_VALUE)
                            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, mainPanelLayout.createSequentialGroup()
                                .addComponent(jPanel1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 13, Short.MAX_VALUE)
                                .addComponent(jPanel2, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                        .addContainerGap())
                    .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, mainPanelLayout.createSequentialGroup()
                        .addComponent(jButton1)
                        .addGap(48, 48, 48)
                        .addComponent(jButton3)
                        .addGap(48, 48, 48)
                        .addComponent(jButton2)
                        .addGap(167, 167, 167))))
        );
        mainPanelLayout.setVerticalGroup(
            mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, mainPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jPanel1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jPanel2, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addGap(18, 18, 18)
                .addComponent(jScrollPane1, javax.swing.GroupLayout.PREFERRED_SIZE, 188, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jButton2)
                    .addComponent(jButton1)
                    .addComponent(jButton3))
                .addContainerGap())
        );

        menuBar.setName("menuBar"); // NOI18N

        fileMenu.setText(resourceMap.getString("fileMenu.text")); // NOI18N
        fileMenu.setName("fileMenu"); // NOI18N

        exitMenuItem.setAction(actionMap.get("quit")); // NOI18N
        exitMenuItem.setName("exitMenuItem"); // NOI18N
        fileMenu.add(exitMenuItem);

        menuBar.add(fileMenu);

        jMenu1.setText(resourceMap.getString("jMenu1.text")); // NOI18N
        jMenu1.setName("jMenu1"); // NOI18N

        jMenuItem2.setAction(actionMap.get("showPlayerDialog")); // NOI18N
        jMenuItem2.setText(resourceMap.getString("jMenuItem2.text")); // NOI18N
        jMenuItem2.setName("jMenuItem2"); // NOI18N
        jMenu1.add(jMenuItem2);

        jMenuItem1.setAction(actionMap.get("showEquipDialog")); // NOI18N
        jMenuItem1.setText(resourceMap.getString("jMenuItem1.text")); // NOI18N
        jMenuItem1.setName("jMenuItem1"); // NOI18N
        jMenu1.add(jMenuItem1);

        menuBar.add(jMenu1);

        helpMenu.setText(resourceMap.getString("helpMenu.text")); // NOI18N
        helpMenu.setName("helpMenu"); // NOI18N

        aboutMenuItem.setAction(actionMap.get("showAboutBox")); // NOI18N
        aboutMenuItem.setName("aboutMenuItem"); // NOI18N
        helpMenu.add(aboutMenuItem);

        menuBar.add(helpMenu);

        setComponent(mainPanel);
        setMenuBar(menuBar);
    }// </editor-fold>//GEN-END:initComponents

    private void jButton1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton1ActionPerformed
         battleSetup();
    }//GEN-LAST:event_jButton1ActionPerformed

    private void jButton3ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton3ActionPerformed
        if (battle != null) battle.battleComplete = true; //stops the battle
    }//GEN-LAST:event_jButton3ActionPerformed

private void battleSetup(){
        battleReportTextArea.setText("");
        int playerSelected = 2;
        String imageFileName = "StauntonKnight2.jpg";
        int weaponSelected = 2;
        int bodyArmourSelected = 2;
        String playerName = "Adamantus";
        // Make the combatants appear
        // set equipment and stats based on selections
        //if user loaded the player dialog, else they'll have default player and name (see above)
        if (playerDialog != null) {
            playerSelected = playerDialog.whichPlayerPicked();
            playerName = playerDialog.getCharacterName();
            if (playerSelected == 1) imageFileName = "StauntonPawn2.jpg";
            if (playerSelected == 3) imageFileName = "Achilles_in_Corfu.jpg";
        }
        //if user loaded the equip dialog, else they'll have default equipment (see above)
        if (equipDialog != null) {
            weaponSelected = equipDialog.whichWeaponPicked();
            bodyArmourSelected = equipDialog.whichBodyArmourPicked();
        }
        //change picture to player image
        ImageIcon icon = new ImageIcon(getClass().getResource(imageFileName)); //has filename
        icon.getImage().flush();
        playerImage.setIcon(icon);
        playerImage.repaint();
        //change other image to enemy image
        ImageIcon iconE = new ImageIcon(getClass().getResource("chimera.png"));
        iconE.getImage().flush();
        enemyImage.setIcon(iconE);
        enemyImage.repaint();
        battle = new BattleController(playerSelected, weaponSelected, bodyArmourSelected, playerName);
        //battle.battleComplete = false;
        updateTextArea();
//        if (BattleController.battleComplete) {
//            battleReportTextArea.append("Okay, that's all for now, folks. \n");
//            BattleController.battleComplete = false;
//        }

       
}

public void updateTextArea() {
        final Runnable checker = new Runnable() {
                public void run() { 
                    while (!battle.battleComplete){
                    textUpdate = battle.battleUpdate;
                    battle.battleUpdate = "";
                    battleReportTextArea.append(textUpdate);
                    battleReportTextArea.setCaretPosition(battleReportTextArea.getDocument().getLength());
                    }
//                    if (battle.battleComplete) scheduler.shutdown();
                    }           
        };        
            
        final ScheduledFuture<?> textUpdateHandle =
            scheduler.scheduleAtFixedRate(checker, 100, 100, MILLISECONDS);
        scheduler.schedule(new Runnable() {
                public void run() { textUpdateHandle.cancel(true); }
            }, 60*60, SECONDS);
    }
    
    //someone said that I'd need to do this for changing the icon to work
//    Runnable setImageIcon = new Runnable() {
//    public void run(){
//        playerProperties = playerDialog.whichPlayerPicked();
//        ImageIcon icon = new ImageIcon(getClass().getResource(playerProperties[6])); //has filename
//        icon.getImage().flush();
//        playerImage.setIcon(icon);
//        playerImage.repaint();
//     }  
//};      
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JTextArea battleReportTextArea;
    private javax.swing.JLabel enemyImage;
    private javax.swing.JButton jButton1;
    private javax.swing.JButton jButton2;
    private javax.swing.JButton jButton3;
    private javax.swing.JMenu jMenu1;
    private javax.swing.JMenuItem jMenuItem1;
    private javax.swing.JMenuItem jMenuItem2;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JPanel mainPanel;
    private javax.swing.JMenuBar menuBar;
    private javax.swing.JLabel playerImage;
    // End of variables declaration//GEN-END:variables

    private final Timer messageTimer;
    private final Timer busyIconTimer;
    private final Icon idleIcon;
    private final Icon[] busyIcons = new Icon[15];
    private int busyIconIndex = 0;

    private JDialog aboutBox;
    //private JDialog equipDialog;
    
    private BattleController battle;
    private String textUpdate;
    private final ScheduledExecutorService scheduler =
       Executors.newScheduledThreadPool(1);
    
    private boolean stopBattle = false;
    
    //put classes 
}
